<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Http;

class TranscriptionController extends Controller
{
    private $apiKey = 'b3a571e932e04521aebbfcc0512702fa';
    private $transcriptEndpoint = 'https://api.assemblyai.com/v2/transcript';

    public function transcribe()
    {
        return view('transcription');
    }

    public function process(Request $request)
    {
        $request->validate([
            'audio_url' => 'required|url'
        ]);

        $audioUrl = $request->input('audio_url');

        // Submit the transcription request
        $response = Http::withHeaders([
            'authorization' => $this->apiKey,
            'content-type' => 'application/json'
        ])->post($this->transcriptEndpoint, [
            'audio_url' => $audioUrl
        ]);

        $transcriptionId = $response->json('id');

        // Poll for completion
        $pollingEndpoint = $this->transcriptEndpoint . '/' . $transcriptionId;
        
        while (true) {
            $pollingResponse = Http::withHeaders([
                'authorization' => $this->apiKey
            ])->get($pollingEndpoint);

            $result = $pollingResponse->json();

            if ($result['status'] === 'completed') {
                return view('transcription', ['transcription' => $result['text'], 'audio_url' => $audioUrl]);
            } elseif ($result['status'] === 'error') {
                return view('transcription', ['error' => 'Transcription failed: ' . $result['error'], 'audio_url' => $audioUrl]);
            }

            sleep(3);
        }
    }
}