<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <meta name="csrf-token" content="<?php echo e(csrf_token()); ?>">
    <title>Chatbot</title>
    <link href="https://maxcdn.bootstrapcdn.com/bootstrap/4.5.2/css/bootstrap.min.css" rel="stylesheet">
    <style>
        .chat-container {
            max-width: 600px;
            margin: 50px auto;
            padding: 20px;
            background-color: #f8f9fa;
            border-radius: 10px;
            box-shadow: 0 0 10px rgba(0,0,0,0.1);
        }
        .chat-box {
            height: 400px;
            overflow-y: scroll;
            margin-bottom: 20px;
            padding: 10px;
            background-color: white;
            border: 1px solid #dee2e6;
            border-radius: 5px;
        }
        .chat-input {
            width: calc(100% - 50px);
        }
        .bot-message {
            text-align: left;
            background-color: #e2e3e5;
            padding: 10px;
            border-radius: 10px;
            margin-bottom: 10px;
            max-width: 75%;
            clear: both;
        }
        .user-message {
            text-align: right;
            background-color: #d4edda;
            padding: 10px;
            border-radius: 10px;
            margin-bottom: 10px;
            max-width: 75%;
            float: right;
        }
        .option-button {
            display: block;
            margin: 5px 0;
        }
        .uploaded-image {
            max-width: 100%;
            height: auto;
            margin-top: 10px;
        }
    </style>
</head>
<body>
    <div class="chat-container">
        <h3 class="text-center">Chat with AI</h3>
        <div class="chat-box" id="chat-box"></div>
        <div class="input-group">
            <input type="text" id="userMessage" class="form-control chat-input" placeholder="Type a message...">
            <div class="input-group-append">
                <button id="sendButton" class="btn btn-primary">Send</button>
            </div>
        </div>
        <div class="input-group mt-3">
            <form id="uploadForm" action="<?php echo e(route('chatbot.upload')); ?>" method="post" enctype="multipart/form-data">
                <?php echo csrf_field(); ?>
                <div class="form-group">
                    <label for="filechoose">Choose File</label>
                    <input type="file" name="file" class="form-control-file" id="filechoose" required>
                    <button class="btn btn-success mt-3" type="button" id="uploadBtn">Upload</button>
                </div>
            </form>
        </div>
        <div id="uploadedImageContainer" class="mt-3" style="display: none;">
            <h5>Uploaded Image:</h5>
            <img id="uploadedImage" class="uploaded-image" src="" alt="Uploaded Image">
        </div>
    </div>
    <script src="https://ajax.googleapis.com/ajax/libs/jquery/3.5.1/jquery.min.js"></script>
    <script>
        $(document).ready(function() {
            // Ensure CSRF token is set for all AJAX requests
            $.ajaxSetup({
                headers: {
                    'X-CSRF-TOKEN': $('meta[name="csrf-token"]').attr('content')
                }
            });

            $('#sendButton').on('click', function() {
                sendMessage();
            });

            $('#userMessage').on('keypress', function(e) {
                if (e.which == 13) {
                    sendMessage();
                }
            });

            $('#uploadBtn').on('click', function() {
                uploadFile();
            });

            function sendMessage() {
                var message = $('#userMessage').val();
                if (message.trim() === '') {
                    return;
                }
                $('#chat-box').append('<div class="user-message">' + message + '</div>');
                $('#userMessage').val('');

                $.ajax({
                    url: '<?php echo e(route("chatbot.send")); ?>',
                    method: 'POST',
                    data: {
                        _token: $('meta[name="csrf-token"]').attr('content'),
                        message: message
                    },
                    success: function(response) {
                        $('#chat-box').append('<div class="bot-message">' + response.message + '</div>');
                        if (response.options) {
                            response.options.forEach(option => {
                                $('#chat-box').append('<button class="btn btn-info option-button">' + option + '</button>');
                            });
                        }
                        $('#chat-box').scrollTop($('#chat-box')[0].scrollHeight);
                    }
                });
            }

            function uploadFile() {
                var formData = new FormData(document.getElementById('uploadForm'));

                $.ajax({
                    url: '<?php echo e(route("chatbot.upload")); ?>',
                    method: 'POST',
                    data: formData,
                    processData: false,
                    contentType: false,
                    success: function(response) {
                        $('#uploadedImage').attr('src', response.uploaded_image_url);
                        $('#uploadedImageContainer').show();
                        $('#chat-box').append('<div class="bot-message">' + response.message + '</div>');
                        $('#chat-box').scrollTop($('#chat-box')[0].scrollHeight);
                        document.getElementById('uploadForm').reset();
                    },
                    error: function(xhr, status, error) {
                        console.error('Error uploading file:', error);
                    }
                });
            }

            $(document).on('click', '.option-button', function() {
                var option = $(this).text();
                $('#userMessage').val(option);
                sendMessage();
                $(this).remove();
            });
        });
    </script>
</body>
</html>
<?php /**PATH C:\xampp\htdocs\dogo\resources\views/chatbot/index.blade.php ENDPATH**/ ?>